var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
var __spreadArray = (this && this.__spreadArray) || function (to, from) {
    for (var i = 0, il = from.length, j = to.length; i < il; i++, j++)
        to[j] = from[i];
    return to;
};
import axios from 'axios';
/**
 * This class handles the http requests using axios
 * @version 1.0.0
 * @author Alejandro <alejandro.devop@gmail.com>
 */
var HttpClient = /** @class */ (function () {
    function HttpClient(options) {
        /** This are the app headers, can be modified using the setter function */
        this.headers = {
            Accept: 'application/json',
            'Content-Type': 'application/json'
        };
        /** Here's where the client stores the requests send */
        this.requestsLog = [];
        /** Flag to control if the application should be logged */
        this.log = false;
        /** The endpoints or paths to make the requests */
        this.endpoints = {};
        var _a = options || {}, auth = _a.auth, server = _a.server, endpoints = _a.endpoints, log = _a.log, onRequestDone = _a.onRequestDone, customHeaders = _a.customHeaders;
        this.server = server;
        this.log = log;
        this.endpoints = endpoints || {};
        this.onRequestDone = onRequestDone;
        if (auth) {
            this.headers['Authorization'] = auth;
        }
        if (customHeaders) {
            this.headers = __assign(__assign({}, this.headers), customHeaders);
        }
    }
    /**
     * Function to get the client headers
     */
    HttpClient.prototype.getHeaders = function () {
        return this.headers;
    };
    /**
     * Function to get the client endpoints
     */
    HttpClient.prototype.getEndpoints = function () {
        return this.endpoints;
    };
    /**
     * Function to get the client server url
     */
    HttpClient.prototype.getServer = function () {
        return this.server;
    };
    /**
     * Function to update the client properties
     * auth: Adds a header to control the authorization
     * @param properties
     */
    HttpClient.prototype.setProperties = function (properties) {
        var auth = (properties || {}).auth;
        this.headers['Authorization'] = auth;
    };
    /**
     * Function to extract paths from the endpoints map
     * @param path
     * @param options
     * @param method
     */
    HttpClient.prototype.resolvePath = function (path, options, method) {
        var _a = options || {}, replace = _a.replace, _b = _a.replaceToken, replaceToken = _b === void 0 ? ':' : _b, pageSize = _a.pageSize, currentPage = _a.currentPage, sorts = _a.sorts;
        var parts = path.split('.');
        var params = [];
        if (pageSize) {
            params.push("page[size]=" + pageSize);
        }
        if (currentPage) {
            params.push("page[number]=" + currentPage);
        }
        if (sorts && sorts.length > 0) {
            params.push("sort=" + sorts.join(','));
        }
        var url = parts.length > 1
            ? parts
                .reduce(function (newValue, currentPart) {
                if (newValue[currentPart]) {
                    newValue = newValue[currentPart];
                }
                return newValue;
            }, this.endpoints)
                .toString()
            : path;
        if (replace) {
            Object.keys(replace).forEach(function (currentKey) {
                url = url.replace(new RegExp("" + replaceToken + currentKey + replaceToken), replace[currentKey]);
            });
        }
        /**
         * If the url has type definition we must clean it
         */
        if (url.includes('<') && url.includes('>')) {
            url = url.replace(new RegExp("<.*>", 'g'), '');
        }
        /**
         * Check if the route has method validation, you can add method validation for paths by prepending
         * [method]:/my/route to your routes, this ensures that no one calls a post method using a get hook
         */
        if (url.includes(':')) {
            var _c = url.split(':'), methodToValidate = _c[0], pathRest = _c[1];
            if (method !== methodToValidate) {
                throw new Error("Invalid method " + method + " for path of type " + methodToValidate);
            }
            else {
                url = pathRest;
            }
        }
        return url.toString() + (params.length > 0 ? "?" + params.join('&') : '');
    };
    /**
     * Function to execute any kind of request
     * @param path
     * @param options
     */
    HttpClient.prototype.doRequest = function (path, options) {
        return __awaiter(this, void 0, void 0, function () {
            var _a, _b, method, params, debug, data, auth, url, response, requestHeaders, requestConfig, e_1;
            return __generator(this, function (_c) {
                switch (_c.label) {
                    case 0:
                        _a = options || {}, _b = _a.method, method = _b === void 0 ? 'get' : _b, params = _a.params, debug = _a.debug, data = _a.data, auth = _a.auth;
                        url = "" + this.server + this.resolvePath(path, options, method);
                        response = null;
                        _c.label = 1;
                    case 1:
                        _c.trys.push([1, 12, , 13]);
                        requestHeaders = __assign(__assign({}, this.headers), (auth ? { Authorization: auth } : {}));
                        requestConfig = {
                            headers: requestHeaders,
                            responseType: 'json'
                        };
                        if (!(method === 'get')) return [3 /*break*/, 3];
                        requestConfig.params = params;
                        return [4 /*yield*/, axios.get(url, requestConfig)];
                    case 2:
                        response = _c.sent();
                        return [3 /*break*/, 11];
                    case 3:
                        if (!(method === 'post')) return [3 /*break*/, 5];
                        return [4 /*yield*/, axios.post(url, data, requestConfig)];
                    case 4:
                        response = _c.sent();
                        return [3 /*break*/, 11];
                    case 5:
                        if (!(method === 'patch')) return [3 /*break*/, 7];
                        return [4 /*yield*/, axios.patch(url, data, requestConfig)];
                    case 6:
                        response = _c.sent();
                        return [3 /*break*/, 11];
                    case 7:
                        if (!(method === 'delete')) return [3 /*break*/, 9];
                        return [4 /*yield*/, axios.delete(url, requestConfig)];
                    case 8:
                        response = _c.sent();
                        return [3 /*break*/, 11];
                    case 9:
                        if (!(method === 'put')) return [3 /*break*/, 11];
                        return [4 /*yield*/, axios.put(url, data, requestConfig)];
                    case 10:
                        response = _c.sent();
                        _c.label = 11;
                    case 11:
                        if (this.log) {
                            this.logRequest({
                                response: response,
                                headers: requestHeaders,
                                url: url,
                                method: method
                            });
                        }
                        if (this.onRequestDone) {
                            this.onRequestDone({
                                response: response,
                                loggedRequests: this.getLog()
                            });
                        }
                        return [2 /*return*/, debug
                                ? {
                                    method: method,
                                    headers: this.headers,
                                    body: data,
                                    data: response,
                                    urlParams: params,
                                    url: url
                                }
                                : response];
                    case 12:
                        e_1 = _c.sent();
                        throw e_1;
                    case 13: return [2 /*return*/];
                }
            });
        });
    };
    HttpClient.prototype.logRequest = function (requestInfo) {
        var method = requestInfo.method, headers = requestInfo.headers, url = requestInfo.url, response = requestInfo.response;
        this.requestsLog = __spreadArray([
            {
                method: method,
                headers: headers,
                url: url,
                response: response
            }
        ], this.requestsLog);
    };
    /**
     * Function to execute a GET request
     * @param path
     * @param options
     */
    HttpClient.prototype.doGet = function (path, options) {
        return __awaiter(this, void 0, void 0, function () {
            var e_2;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        _a.trys.push([0, 2, , 3]);
                        return [4 /*yield*/, this.doRequest(path, __assign(__assign({}, options), { method: 'get' }))];
                    case 1: return [2 /*return*/, _a.sent()];
                    case 2:
                        e_2 = _a.sent();
                        throw e_2;
                    case 3: return [2 /*return*/];
                }
            });
        });
    };
    /**
     * Function to execute a POST request
     * @param path
     * @param options
     */
    HttpClient.prototype.doPost = function (path, options) {
        return __awaiter(this, void 0, void 0, function () {
            var e_3;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        _a.trys.push([0, 2, , 3]);
                        return [4 /*yield*/, this.doRequest(path, __assign(__assign({}, options), { method: 'post' }))];
                    case 1: return [2 /*return*/, _a.sent()];
                    case 2:
                        e_3 = _a.sent();
                        throw e_3;
                    case 3: return [2 /*return*/];
                }
            });
        });
    };
    /**
     * Function to execute a PATCH request
     * @param path
     * @param options
     */
    HttpClient.prototype.doPatch = function (path, options) {
        return __awaiter(this, void 0, void 0, function () {
            var e_4;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        _a.trys.push([0, 2, , 3]);
                        return [4 /*yield*/, this.doRequest(path, __assign(__assign({}, options), { method: 'patch' }))];
                    case 1: return [2 /*return*/, _a.sent()];
                    case 2:
                        e_4 = _a.sent();
                        throw e_4;
                    case 3: return [2 /*return*/];
                }
            });
        });
    };
    /**
     * Function to execute a PUT
     * @param path
     * @param options
     */
    HttpClient.prototype.doPut = function (path, options) {
        return __awaiter(this, void 0, void 0, function () {
            var e_5;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        _a.trys.push([0, 2, , 3]);
                        return [4 /*yield*/, this.doRequest(path, __assign(__assign({}, options), { method: 'put' }))];
                    case 1: return [2 /*return*/, _a.sent()];
                    case 2:
                        e_5 = _a.sent();
                        throw e_5;
                    case 3: return [2 /*return*/];
                }
            });
        });
    };
    /**
     * Function to execute a Delete Request
     * @param path
     * @param options
     */
    HttpClient.prototype.doDelete = function (path, options) {
        return __awaiter(this, void 0, void 0, function () {
            var e_6;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        _a.trys.push([0, 2, , 3]);
                        return [4 /*yield*/, this.doRequest(path, __assign(__assign({}, options), { method: 'delete' }))];
                    case 1: return [2 /*return*/, _a.sent()];
                    case 2:
                        e_6 = _a.sent();
                        throw e_6;
                    case 3: return [2 /*return*/];
                }
            });
        });
    };
    /**
     * Function to retrieve the application requests log
     */
    HttpClient.prototype.getLog = function () {
        return this.requestsLog;
    };
    HttpClient.prototype.setEndpoints = function (endpoints) {
        this.endpoints = endpoints;
    };
    return HttpClient;
}());
export default HttpClient;
//# sourceMappingURL=HttpClient.js.map